<?php

namespace Teknisa\CADSUP\FloatingReportGenerator;

class ReportGenerator {
    
    const ROUTES_TO_VERIFY = [
            '/generateReport',
            '/generateXLS'
        ];

    const METHOD_BEFORE_GENERATE = "beforeGenerateReport";
    const METHOD_AFTER_GENERATE = "afterGenerateReport";
    
    private $classReportGeneratorByWidget;
    
    public function __construct($classReportGeneratorByWidget){
        
        $this->classReportGeneratorByWidget = $classReportGeneratorByWidget;
        
    }
    
    public function run($request, $response = false) {
        
        $route = $request->getRoutePath();
        
        if($this->isRouteToValidate($route)){
            $parameters = $request->getParameters();
            $metaData = $parameters['metaData'] ?? null;
            $widgetName = $metaData['name'] ?? null;            
            $classByWidgetName = $this->getClassByWidgetName($widgetName);
            $method = $response === false ? self::METHOD_BEFORE_GENERATE : self::METHOD_AFTER_GENERATE;
            
            $methodIsValid = $this->isValidMethod($classByWidgetName, $method);            
            
            if($methodIsValid){
                try{
                    call_user_func_array([$classByWidgetName, $method], [$request, $response]);
                }catch(\Exception $e){
                    var_dump($e->getMessage());die;
                    return $e->getMessage();
                }
            }
        }
    }
    
    private function isValidMethod ($classInstance, $methodName) {
        
        return method_exists($classInstance, $methodName) && is_callable([$classInstance, $methodName]);
    } 
    
    private function getClassByWidgetName($widgetName) {
        
        $classByWidgetName = $this->classReportGeneratorByWidget[$widgetName] ?? null;
        
        return $classByWidgetName;
    }
    
    private function isRouteToValidate($route) {
        
        return array_search($route, self::ROUTES_TO_VERIFY)!== false;
        
    }
}